/*
 * Copyright 2000, International Business Machines Corporation and others.
 * All Rights Reserved.
 * 
 * This software has been released under the terms of the IBM Public
 * License.  For details, see the LICENSE file in the top-level source
 * directory or online at http://www.openafs.org/dl/license10.html
 */

#include "al_resource.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#include <windows.h>
#include <commctrl.h>
#ifndef APSTUDIO_INVOKED
#define AFS_VERINFO_FILE_DESCRIPTION "AFS 응용프로그램 기본 DLL"
#include "AFS_component_version_number.h"
#include "..\..\config\NTVersioninfo.rc"
#endif // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// Korean resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_KOR)
#ifdef _WIN32
LANGUAGE LANG_KOREAN, SUBLANG_DEFAULT
#endif //_WIN32

#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE DISCARDABLE 
BEGIN
    "al_resource.h\0"
END

2 TEXTINCLUDE DISCARDABLE 
BEGIN
    "#include <windows.h>\r\n"
    "#include <commctrl.h>\r\n"
    "#ifndef APSTUDIO_INVOKED\r\n"
    "#define AFS_VERINFO_FILE_DESCRIPTION ""AFS 응용프로그램 기본 DLL""\r\n"
    "#include ""AFS_component_version_number.h""\r\n"
    "#include ""..\\..\\config\\NTVersioninfo.rc""\r\n"
    "#endif // APSTUDIO_INVOKED\r\n"
    "\r\n"
    "\0"
END

3 TEXTINCLUDE DISCARDABLE 
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
32513                   ICON    DISCARDABLE     "Resource\\error.ico"
32515                   ICON    DISCARDABLE     "Resource\\warning.ico"
IDI_SPIN1               ICON    DISCARDABLE     "resource\\spin1.ico"
IDI_SPIN2               ICON    DISCARDABLE     "resource\\spin2.ico"
IDI_SPIN3               ICON    DISCARDABLE     "resource\\spin3.ico"
IDI_SPIN4               ICON    DISCARDABLE     "resource\\spin4.ico"
IDI_SPIN5               ICON    DISCARDABLE     "resource\\spin5.ico"
IDI_SPIN6               ICON    DISCARDABLE     "resource\\spin6.ico"
IDI_SPIN7               ICON    DISCARDABLE     "resource\\spin7.ico"
IDI_SPIN8               ICON    DISCARDABLE     "resource\\spin8.ico"
IDI_SPINSTOP            ICON    DISCARDABLE     "resource\\spinstop.ico"
IDI_BOSSERVICE          ICON    DISCARDABLE     "resource\\bos.ico"
IDI_SERVER              ICON    DISCARDABLE     "resource\\server.ico"
IDI_SERVER_ALERT        ICON    DISCARDABLE     "resource\\server1.ico"
IDI_SERVER_UNMON        ICON    DISCARDABLE     "resource\\server_u.ico"
IDI_SERVICE             ICON    DISCARDABLE     "resource\\service.ico"
IDI_SERVICE_ALERT       ICON    DISCARDABLE     "resource\\service1.ico"
IDI_SERVICE_STOPPED     ICON    DISCARDABLE     "resource\\service2.ico"
IDI_AGGREGATE           ICON    DISCARDABLE     "resource\\aggreg.ico"
IDI_AGGREGATE_ALERT     ICON    DISCARDABLE     "resource\\aggreg1.ico"
IDI_FILESET             ICON    DISCARDABLE     "resource\\fileset.ico"
IDI_FILESET_ALERT       ICON    DISCARDABLE     "resource\\fileset1.ico"
IDI_FILESET_LOCKED      ICON    DISCARDABLE     "resource\\locked.ico"
IDI_INITIAL_CONTAINER   ICON    DISCARDABLE     "resource\\ic.ico"
IDI_INITIAL_OBJECT      ICON    DISCARDABLE     "resource\\io.ico"
IDI_MULTIPLE            ICON    DISCARDABLE     "resource\\multiple.ico"
IDI_ACL                 ICON    DISCARDABLE     "resource\\acl.ico"
IDI_CELL                ICON    DISCARDABLE     "resource\\cell.ico"
IDI_SERVER_KEY          ICON    DISCARDABLE     "resource\\key.ico"
IDI_USER                ICON    DISCARDABLE     "resource\\user.ico"
IDI_GROUP               ICON    DISCARDABLE     "resource\\group.ico"

/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

IDD_APPLIB_ERROR DIALOG DISCARDABLE  0, 0, 277, 91
STYLE DS_MODALFRAME | DS_CENTER | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "오류"
FONT 9, "굴림"
BEGIN
    LTEXT           "Windows는 인식할 수 없는 오류로 인하여 요청된 작업을 종료할 수 없습니다.",
                    IDC_ERROR_DESC,31,7,242,33
    LTEXT           "오류: %1",IDC_ERROR_STATUS,31,49,241,16
    DEFPUSHBUTTON   "확인",IDOK,114,73,50,14
    ICON            32513,IDC_STATIC,4,7,20,20
END

IDD_APPLIB_OPENCELL DIALOG DISCARDABLE  0, 0, 219, 164
STYLE DS_MODALFRAME | DS_CENTER | DS_CONTEXTHELP | WS_POPUP | WS_CAPTION | 
    WS_SYSMENU
CAPTION "셀 선택"
FONT 9, "굴림"
BEGIN
    LTEXT           "셀 (&C):",IDC_STATIC,4,24,16,8
    LTEXT           "작업할 AFS 셀을 선택하십시오.",IDC_STATIC,4,7,200,8
    COMBOBOX        IDC_OPENCELL_CELL,24,22,191,143,CBS_DROPDOWN | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "AFS ID (&I):",IDC_STATIC,22,100,53,8
    EDITTEXT        IDC_OPENCELL_ID,80,98,123,13,ES_AUTOHSCROLL
    LTEXT           "AFS 암호 (&P):",IDC_STATIC,22,116,55,8
    EDITTEXT        IDC_OPENCELL_PASSWORD,80,114,124,13,ES_PASSWORD | 
                    ES_AUTOHSCROLL
    DEFPUSHBUTTON   "확인",IDOK,57,146,50,14
    PUSHBUTTON      "취소",IDCANCEL,111,146,50,14
    PUSHBUTTON      "도움말(&H)",9,165,146,50,14
    GROUPBOX        "AFS 인증",IDC_STATIC,4,44,211,92
    LTEXT           "(조회 중...)",IDC_OPENCELL_OLDCREDS,22,70,185,8
    LTEXT           "셀을 관리하려면, AFS 암호를 다시 입력해야 합니다.",
                    IDC_STATIC,10,85,196,8
    LTEXT           "선택된 셀에 있는 현재 AFS 토큰:",IDC_STATIC,10,57,196,8
END

IDD_APPLIB_CREDENTIALS DIALOGEX 0, 0, 215, 159
STYLE DS_MODALFRAME | DS_CENTER | DS_CONTEXTHELP | WS_CAPTION | WS_SYSMENU
CAPTION "AFS 인증"
FONT 9, "굴림"
BEGIN
    LTEXT           "AFS ID (&I):",IDC_STATIC,11,97,51,8
    EDITTEXT        IDC_CREDS_ID,67,96,124,14,ES_AUTOHSCROLL
    LTEXT           "AFS 암호 (&P):",IDC_STATIC,11,114,50,8
    EDITTEXT        IDC_CREDS_PASSWORD,67,111,124,14,ES_PASSWORD | 
                    ES_AUTOHSCROLL
    LTEXT           "AFS 셀:",IDC_STATIC,4,7,52,8
    EDITTEXT        IDC_CREDS_CELL,75,7,129,12,ES_AUTOHSCROLL | ES_READONLY | 
                    NOT WS_BORDER | NOT WS_TABSTOP,WS_EX_STATICEDGE
    LTEXT           "현재 ID:",IDC_STATIC,4,23,52,8
    EDITTEXT        IDC_CREDS_CURRENTID,75,22,129,12,ES_AUTOHSCROLL | 
                    ES_READONLY | NOT WS_BORDER | NOT WS_TABSTOP,
                    WS_EX_STATICEDGE
    LTEXT           "만기 날짜:",IDC_STATIC,4,39,52,8
    EDITTEXT        IDC_CREDS_EXPDATE,75,38,129,12,ES_AUTOHSCROLL | 
                    ES_READONLY | NOT WS_BORDER | NOT WS_TABSTOP,
                    WS_EX_STATICEDGE
    DEFPUSHBUTTON   "확인",IDOK,35,141,50,14
    PUSHBUTTON      "취소",IDCANCEL,89,141,50,14
    PUSHBUTTON      "도움말(&H)",9,161,141,50,14
    GROUPBOX        "   ",IDC_STATIC,4,68,207,70
    CONTROL         "이 셀에 대한 새 AFS 토큰 확보 (&O)",IDC_CREDS_LOGIN,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,10,81,138,10
END

IDD_APPLIB_BADCREDS DIALOG DISCARDABLE  0, 0, 335, 96
STYLE DS_MODALFRAME | DS_CENTER | DS_CONTEXTHELP | WS_POPUP | WS_CAPTION | 
    WS_SYSMENU
CAPTION "주의"
FONT 9, "굴림"
BEGIN
    DEFPUSHBUTTON   "예 (&Y)",IDOK,173,78,50,14
    PUSHBUTTON      "아니오 (&N)",IDCANCEL,227,78,50,14
    PUSHBUTTON      "도움말(&H)",9,281,78,50,14
    CONTROL         "이 경고를 다시 표시하지 않음(&D)",IDC_BADCREDS_SHUTUP,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,31,80,120,10
    ICON            32515,IDC_STATIC,4,7,20,20
    LTEXT           "(실행 중 설정된 텍스트)",IDC_BADCREDS_DESC,31,7,300,64
END

IDD_APPLIB_COVER DIALOG DISCARDABLE  0, 0, 189, 135
STYLE WS_CHILD | WS_BORDER
EXSTYLE WS_EX_CLIENTEDGE
FONT 9, "굴림"
BEGIN
    GROUPBOX        "",IDC_COVER_BORDER,7,4,175,123
    LTEXT           "(설명은 여기에 옵니다)",IDC_COVER_DESC,14,14,162,107
    PUSHBUTTON      "다시 시도하십시오 (&A)...",IDC_COVER_BUTTON,126,107,50,
                    14
END

IDD_APPLIB_BROWSE_FILESET DIALOGEX 0, 0, 281, 150
STYLE DS_MODALFRAME | DS_CONTEXTHELP | WS_POPUP | WS_VISIBLE | 
    WS_CLIPCHILDREN | WS_CAPTION | WS_SYSMENU
CAPTION "볼륨 선택"
FONT 9, "굴림"
BEGIN
    RTEXT           "볼륨:",IDC_BROWSE_TYPE,7,115,49,8,SS_NOTIFY
    EDITTEXT        IDC_BROWSE_NAMED,60,113,145,14,ES_MULTILINE | 
                    ES_AUTOVSCROLL | ES_AUTOHSCROLL | ES_WANTRETURN
    PUSHBUTTON      "선택(&S)",IDOK,224,113,50,14
    PUSHBUTTON      "취소",IDCANCEL,224,129,50,14
    RTEXT           "셀 조회(&C):",IDC_STATIC,7,9,41,8,SS_NOTIFY
    COMBOBOX        IDC_BROWSE_CELL,56,7,153,91,CBS_DROPDOWN | 
                    CBS_AUTOHSCROLL | CBS_SORT | CBS_HASSTRINGS | WS_VSCROLL | 
                    WS_TABSTOP
    DEFPUSHBUTTON   "(실행 중에 표시되지 않음)",IDC_BROWSE_RESTART,41,130,
                    177,13,NOT WS_TABSTOP
    CONTROL         "",IDC_BROWSE_LIST,"SysListView32",LVS_LIST | 
                    LVS_SINGLESEL | LVS_ALIGNLEFT | LVS_NOCOLUMNHEADER | 
                    WS_BORDER | WS_TABSTOP,7,23,267,84,WS_EX_CLIENTEDGE
END

IDD_APPLIB_BROWSE DIALOG DISCARDABLE  0, 0, 281, 150
STYLE DS_MODALFRAME | WS_POPUP | WS_VISIBLE | WS_CLIPCHILDREN | WS_CAPTION | 
    WS_SYSMENU
CAPTION "(실행 중 선택된 제목)"
FONT 9, "굴림"
BEGIN
    RTEXT           "(실행 중)",IDC_BROWSE_TYPE,7,115,49,8,SS_NOTIFY
    EDITTEXT        IDC_BROWSE_NAMED,60,113,145,14,ES_MULTILINE | 
                    ES_AUTOVSCROLL | ES_AUTOHSCROLL | ES_WANTRETURN
    PUSHBUTTON      "선택(&S)",IDC_BROWSE_SELECT,224,112,50,14
    PUSHBUTTON      "취소",IDCANCEL,224,129,50,14
    RTEXT           "셀 조회(&C):",IDC_STATIC,7,9,41,8,SS_NOTIFY
    COMBOBOX        IDC_BROWSE_CELL,56,7,153,91,CBS_DROPDOWN | 
                    CBS_AUTOHSCROLL | CBS_SORT | CBS_HASSTRINGS | WS_VSCROLL | 
                    WS_TABSTOP
    CONTROL         "",IDC_BROWSE_LIST,"SysListView32",LVS_LIST | 
                    LVS_SINGLESEL | LVS_ALIGNLEFT | LVS_NOCOLUMNHEADER | 
                    WS_BORDER | WS_TABSTOP,7,23,267,84,WS_EX_CLIENTEDGE
    DEFPUSHBUTTON   "(실행 중에 표시되지 않음)",IDC_BROWSE_RESTART,43,137,
                    177,13,NOT WS_TABSTOP
    CTEXT           "",IDC_BROWSE_STATUS,56,133,153,8,SS_NOTIFY
END


/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO DISCARDABLE 
BEGIN
    IDD_APPLIB_ERROR, DIALOG
    BEGIN
        LEFTMARGIN, 4
        RIGHTMARGIN, 273
        TOPMARGIN, 7
        BOTTOMMARGIN, 87
    END

    IDD_APPLIB_OPENCELL, DIALOG
    BEGIN
        LEFTMARGIN, 4
        RIGHTMARGIN, 215
        TOPMARGIN, 7
        BOTTOMMARGIN, 160
    END

    IDD_APPLIB_CREDENTIALS, DIALOG
    BEGIN
        LEFTMARGIN, 4
        RIGHTMARGIN, 211
        TOPMARGIN, 7
        BOTTOMMARGIN, 155
    END

    IDD_APPLIB_BADCREDS, DIALOG
    BEGIN
        LEFTMARGIN, 4
        RIGHTMARGIN, 331
        TOPMARGIN, 7
        BOTTOMMARGIN, 92
    END

    IDD_APPLIB_COVER, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 182
        TOPMARGIN, 7
        BOTTOMMARGIN, 128
    END

    IDD_APPLIB_BROWSE_FILESET, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 274
        TOPMARGIN, 7
        BOTTOMMARGIN, 143
        HORZGUIDE, 23
        HORZGUIDE, 107
    END

    IDD_APPLIB_BROWSE, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 274
        TOPMARGIN, 7
        BOTTOMMARGIN, 143
        HORZGUIDE, 23
        HORZGUIDE, 107
    END
END
#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Bitmap
//

IDB_LOCAL_USER          BITMAP  DISCARDABLE     "resource\\local_user.bmp"
IDB_LOCAL_GROUP         BITMAP  DISCARDABLE     "resource\\local_group.bmp"
IDB_FOREIGN_USER        BITMAP  DISCARDABLE     "resource\\foreign_user.bmp"
IDB_FOREIGN_GROUP       BITMAP  DISCARDABLE     "resource\\foreign_group.bmp"
IDB_ANY_OTHER           BITMAP  DISCARDABLE     "resource\\any_other.bmp"
IDB_FOREIGN_OTHER       BITMAP  DISCARDABLE     "resource\\foreign_other.bmp"
IDB_LOCAL_OTHER         BITMAP  DISCARDABLE     "resource\\local_other.bmp"
IDB_MASK_OBJ            BITMAP  DISCARDABLE     "resource\\mask_obj.bmp"
IDB_UNAUTHENTICATED     BITMAP  DISCARDABLE     "resource\\unauthenticated.bmp"

/////////////////////////////////////////////////////////////////////////////
//
// String Table
//

STRINGTABLE DISCARDABLE 
BEGIN
    IDS_BADCREDS_DESC_GENERAL 
                            "%1 셀에서 AFS 토큰을 관리자로 사용하지 않습니다.\n\n%2\n\n%3"
    IDS_BADCREDS_DESC_EXPIRED "%1 셀의 토큰이 만기되었습니다.\n\n%2\n\n%3"
    IDS_BADCREDS_DESC_BADCHOICE 
                            "%1 셀에서 관리자로 로그인하지 않기로 결정했습니다.\n\n%2\n\n%3"
    IDS_BADCREDS_DESC2      "관리자로 로그인하지 않는 이상, %1 셀과 작업할 수 있는 특권이 부족합니다."
    IDS_BADCREDS_DESC3      "지금 새 AFS 토큰을 확보하시겠습니까?"
    IDS_BADCREDS_DESC3_ADM  "관리자로 로그인하시겠습니까?"
    IDS_BADCREDS_DESC_DESTROYED 
                            "%1 셀의 토큰은 더 이상 유효하지 않습니다.\n\n%2\n\n%3"
END

STRINGTABLE DISCARDABLE 
BEGIN
    IDS_ERROR_BAD_CREDENTIALS 
                            "입력한 AFS ID를 인식할 수 없거나 지정한 암호가 틀렸습니다."
    IDS_BROWSE_WAITING      "조회 중입니다. 잠시 기다리십시오..."
    IDS_BROWSE_BADCELL_TITLE "인식되지 않은 셀"
    IDS_BROWSE_BADCELL_DESC "입력한 셀 이름을 네트워크에서 찾을 수 없습니다.\n\n셀 이름을 확인하여 올바로 입력했는지 보십시오."
    IDS_CREDS_NONE          "(토큰 없음)"
    IDS_CREDS_VALID         "%2, %3 만기"
    IDS_CREDS_EXPIRED       "(토큰 %3 만기)"
END

#endif    // Korean resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

