/*
 * Copyright 2000, International Business Machines Corporation and others.
 * All Rights Reserved.
 * 
 * This software has been released under the terms of the IBM Public
 * License.  For details, see the LICENSE file in the top-level source
 * directory or online at http://www.openafs.org/dl/license10.html
 */

#include "al_resource.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#include <windows.h>
#include <commctrl.h>
#ifndef APSTUDIO_INVOKED
#define AFS_VERINFO_FILE_DESCRIPTION "AFS Application Base DLL"
#include "AFS_component_version_number.h"
#include "..\..\config\NTVersioninfo.rc"
#endif // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// English (U.S.) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
#ifdef _WIN32
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
#pragma code_page(1252)
#endif //_WIN32

#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE DISCARDABLE 
BEGIN
    "al_resource.h\0"
END

2 TEXTINCLUDE DISCARDABLE 
BEGIN
    "#include <windows.h>\r\n"
    "#include <commctrl.h>\r\n"
    "#ifndef APSTUDIO_INVOKED\r\n"
    "#define AFS_VERINFO_FILE_DESCRIPTION ""AFS Application Base DLL""\r\n"
    "#include ""AFS_component_version_number.h""\r\n"
    "#include ""..\\..\\config\\NTVersioninfo.rc""\r\n"
    "#endif // APSTUDIO_INVOKED\r\n"
    "\r\n"
    "\0"
END

3 TEXTINCLUDE DISCARDABLE 
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
32513                   ICON    DISCARDABLE     "Resource\\error.ico"
32515                   ICON    DISCARDABLE     "Resource\\warning.ico"
IDI_SPIN1               ICON    DISCARDABLE     "resource\\spin1.ico"
IDI_SPIN2               ICON    DISCARDABLE     "resource\\spin2.ico"
IDI_SPIN3               ICON    DISCARDABLE     "resource\\spin3.ico"
IDI_SPIN4               ICON    DISCARDABLE     "resource\\spin4.ico"
IDI_SPIN5               ICON    DISCARDABLE     "resource\\spin5.ico"
IDI_SPIN6               ICON    DISCARDABLE     "resource\\spin6.ico"
IDI_SPIN7               ICON    DISCARDABLE     "resource\\spin7.ico"
IDI_SPIN8               ICON    DISCARDABLE     "resource\\spin8.ico"
IDI_SPINSTOP            ICON    DISCARDABLE     "resource\\spinstop.ico"
IDI_BOSSERVICE          ICON    DISCARDABLE     "resource\\bos.ico"
IDI_SERVER              ICON    DISCARDABLE     "resource\\server.ico"
IDI_SERVER_ALERT        ICON    DISCARDABLE     "resource\\server1.ico"
IDI_SERVER_UNMON        ICON    DISCARDABLE     "resource\\server_u.ico"
IDI_SERVICE             ICON    DISCARDABLE     "resource\\service.ico"
IDI_SERVICE_ALERT       ICON    DISCARDABLE     "resource\\service1.ico"
IDI_SERVICE_STOPPED     ICON    DISCARDABLE     "resource\\service2.ico"
IDI_AGGREGATE           ICON    DISCARDABLE     "resource\\aggreg.ico"
IDI_AGGREGATE_ALERT     ICON    DISCARDABLE     "resource\\aggreg1.ico"
IDI_FILESET             ICON    DISCARDABLE     "resource\\fileset.ico"
IDI_FILESET_ALERT       ICON    DISCARDABLE     "resource\\fileset1.ico"
IDI_FILESET_LOCKED      ICON    DISCARDABLE     "resource\\locked.ico"
IDI_INITIAL_CONTAINER   ICON    DISCARDABLE     "resource\\ic.ico"
IDI_INITIAL_OBJECT      ICON    DISCARDABLE     "resource\\io.ico"
IDI_MULTIPLE            ICON    DISCARDABLE     "resource\\multiple.ico"
IDI_ACL                 ICON    DISCARDABLE     "resource\\acl.ico"
IDI_CELL                ICON    DISCARDABLE     "resource\\cell.ico"
IDI_SERVER_KEY          ICON    DISCARDABLE     "resource\\key.ico"
IDI_USER                ICON    DISCARDABLE     "resource\\user.ico"
IDI_GROUP               ICON    DISCARDABLE     "resource\\group.ico"

/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

IDD_APPLIB_ERROR DIALOG DISCARDABLE  0, 0, 277, 91
STYLE DS_MODALFRAME | DS_CENTER | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Error"
FONT 9, "MS Shell Dlg"
BEGIN
    LTEXT           "Windows was unable to complete the requested task because an unrecognized error occurred.",
                    IDC_ERROR_DESC,31,7,242,33
    LTEXT           "Error: %1",IDC_ERROR_STATUS,31,49,241,16
    DEFPUSHBUTTON   "OK",IDOK,114,73,50,14
    ICON            32513,IDC_STATIC,4,7,20,20
END

IDD_APPLIB_OPENCELL DIALOG DISCARDABLE  0, 0, 219, 164
STYLE DS_MODALFRAME | DS_CENTER | DS_CONTEXTHELP | WS_POPUP | WS_CAPTION | 
    WS_SYSMENU
CAPTION "Select Cell"
FONT 9, "MS Shell Dlg"
BEGIN
    LTEXT           "&Cell:",IDC_STATIC,4,24,16,8
    LTEXT           "Select the AFS cell with which you would like to work.",
                    IDC_STATIC,4,7,200,8
    COMBOBOX        IDC_OPENCELL_CELL,24,22,191,143,CBS_DROPDOWN | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "AFS &Identity:",IDC_STATIC,22,100,53,8
    EDITTEXT        IDC_OPENCELL_ID,80,98,128,13,ES_AUTOHSCROLL
    LTEXT           "AFS &Password:",IDC_STATIC,22,116,55,8
    EDITTEXT        IDC_OPENCELL_PASSWORD,80,114,128,13,ES_PASSWORD | 
                    ES_AUTOHSCROLL
    DEFPUSHBUTTON   "OK",IDOK,57,146,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,111,146,50,14
    PUSHBUTTON      "&Help",9,165,146,50,14
    GROUPBOX        "AFS Authentication",IDC_STATIC,4,44,211,92
    LTEXT           "(querying...)",IDC_OPENCELL_OLDCREDS,22,70,185,8
    LTEXT           "To administer a cell, you must re-enter your AFS password:",
                    IDC_STATIC,10,85,196,8
    LTEXT           "Your current AFS tokens within the selected cell:",
                    IDC_STATIC,10,57,196,8
END

IDD_APPLIB_CREDENTIALS DIALOGEX 0, 0, 197, 140
STYLE DS_MODALFRAME | DS_CENTER | DS_CONTEXTHELP | WS_CAPTION | WS_SYSMENU
CAPTION "AFS Authentication"
FONT 9, "MS Shell Dlg"
BEGIN
    LTEXT           "AFS &Identity:",IDC_STATIC,8,75,51,8
    EDITTEXT        IDC_CREDS_ID,64,73,124,14,ES_AUTOHSCROLL
    LTEXT           "AFS &Password:",IDC_STATIC,8,92,50,8
    EDITTEXT        IDC_CREDS_PASSWORD,64,89,124,14,ES_PASSWORD | 
                    ES_AUTOHSCROLL
    LTEXT           "AFS Cell:",IDC_STATIC,4,7,52,8
    EDITTEXT        IDC_CREDS_CELL,64,7,129,12,ES_AUTOHSCROLL | ES_READONLY | 
                    NOT WS_BORDER | NOT WS_TABSTOP,WS_EX_STATICEDGE
    LTEXT           "Current Identity:",IDC_STATIC,4,23,52,8
    EDITTEXT        IDC_CREDS_CURRENTID,64,22,129,12,ES_AUTOHSCROLL | 
                    ES_READONLY | NOT WS_BORDER | NOT WS_TABSTOP,
                    WS_EX_STATICEDGE
    LTEXT           "Expiration Date:",IDC_STATIC,4,39,52,8
    EDITTEXT        IDC_CREDS_EXPDATE,64,38,129,12,ES_AUTOHSCROLL | 
                    ES_READONLY | NOT WS_BORDER | NOT WS_TABSTOP,
                    WS_EX_STATICEDGE
    DEFPUSHBUTTON   "OK",IDOK,35,122,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,89,122,50,14
    PUSHBUTTON      "&Help",9,143,122,50,14
    GROUPBOX        "   ",IDC_STATIC,4,57,189,56
    CONTROL         "&Obtain new AFS tokens for this cell",IDC_CREDS_LOGIN,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,8,57,138,10
END

IDD_APPLIB_BADCREDS DIALOG DISCARDABLE  0, 0, 335, 96
STYLE DS_MODALFRAME | DS_CENTER | DS_CONTEXTHELP | WS_POPUP | WS_CAPTION | 
    WS_SYSMENU
CAPTION "Caution"
FONT 9, "MS Shell Dlg"
BEGIN
    DEFPUSHBUTTON   "&Yes",IDOK,173,78,50,14
    PUSHBUTTON      "&No",IDCANCEL,227,78,50,14
    PUSHBUTTON      "&Help",9,281,78,50,14
    CONTROL         "&Do not show this warning again",IDC_BADCREDS_SHUTUP,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,31,80,120,10
    ICON            32515,IDC_STATIC,4,7,20,20
    LTEXT           "(text set at runtime)",IDC_BADCREDS_DESC,31,7,300,64
END

IDD_APPLIB_COVER DIALOG DISCARDABLE  0, 0, 189, 135
STYLE WS_CHILD | WS_BORDER
EXSTYLE WS_EX_CLIENTEDGE
FONT 9, "MS Shell Dlg"
BEGIN
    GROUPBOX        "",IDC_COVER_BORDER,7,4,175,123
    LTEXT           "(description goes here)",IDC_COVER_DESC,14,14,162,107
    PUSHBUTTON      "Try &Again...",IDC_COVER_BUTTON,126,107,50,14
END

IDD_APPLIB_BROWSE_FILESET DIALOGEX 0, 0, 281, 150
STYLE DS_MODALFRAME | DS_CONTEXTHELP | WS_POPUP | WS_VISIBLE | 
    WS_CLIPCHILDREN | WS_CAPTION | WS_SYSMENU
CAPTION "Select Volume"
FONT 9, "MS Shell Dlg", 0, 0, 0x1
BEGIN
    RTEXT           "Volume:",IDC_BROWSE_TYPE,7,115,49,8,SS_NOTIFY
    EDITTEXT        IDC_BROWSE_NAMED,60,113,145,14,ES_MULTILINE | 
                    ES_AUTOVSCROLL | ES_AUTOHSCROLL | ES_WANTRETURN
    PUSHBUTTON      "&Select",IDOK,224,113,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,224,129,50,14
    RTEXT           "Look in &cell:",IDC_STATIC,7,9,41,8,SS_NOTIFY
    COMBOBOX        IDC_BROWSE_CELL,56,7,153,91,CBS_DROPDOWN | 
                    CBS_AUTOHSCROLL | CBS_SORT | CBS_HASSTRINGS | WS_VSCROLL | 
                    WS_TABSTOP
    CONTROL         "",IDC_BROWSE_LIST,"SysListView32",LVS_LIST | 
                    LVS_SINGLESEL | LVS_ALIGNLEFT | LVS_NOCOLUMNHEADER | 
                    WS_BORDER | WS_TABSTOP,7,23,267,84,WS_EX_CLIENTEDGE
    CTEXT           "",IDC_BROWSE_STATUS,56,133,153,8,SS_NOTIFY
    DEFPUSHBUTTON   "(I get moved off the dialog so I'm invisible at runtime)",
                    IDC_BROWSE_RESTART,43,137,177,13,NOT WS_TABSTOP
END

IDD_APPLIB_BROWSE DIALOG DISCARDABLE  0, 0, 281, 150
STYLE DS_MODALFRAME | WS_POPUP | WS_VISIBLE | WS_CLIPCHILDREN | WS_CAPTION | 
    WS_SYSMENU
CAPTION "(This title selected at runtime)"
FONT 9, "MS Shell Dlg"
BEGIN
    RTEXT           "(runtime)",IDC_BROWSE_TYPE,7,115,49,8,SS_NOTIFY
    EDITTEXT        IDC_BROWSE_NAMED,60,113,145,14,ES_MULTILINE | 
                    ES_AUTOVSCROLL | ES_AUTOHSCROLL | ES_WANTRETURN
    PUSHBUTTON      "&Select",IDC_BROWSE_SELECT,224,112,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,224,129,50,14
    RTEXT           "Look in &cell:",IDC_STATIC,7,9,41,8,SS_NOTIFY
    COMBOBOX        IDC_BROWSE_CELL,56,7,153,91,CBS_DROPDOWN | 
                    CBS_AUTOHSCROLL | CBS_SORT | CBS_HASSTRINGS | WS_VSCROLL | 
                    WS_TABSTOP
    CONTROL         "",IDC_BROWSE_LIST,"SysListView32",LVS_LIST | 
                    LVS_SINGLESEL | LVS_ALIGNLEFT | LVS_NOCOLUMNHEADER | 
                    WS_BORDER | WS_TABSTOP,7,23,267,84,WS_EX_CLIENTEDGE
    DEFPUSHBUTTON   "(I get moved off the dialog so I'm invisible at runtime)",
                    IDC_BROWSE_RESTART,43,137,177,13,NOT WS_TABSTOP
    CTEXT           "",IDC_BROWSE_STATUS,56,133,153,8,SS_NOTIFY
END


/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO DISCARDABLE 
BEGIN
    IDD_APPLIB_ERROR, DIALOG
    BEGIN
        LEFTMARGIN, 4
        RIGHTMARGIN, 273
        TOPMARGIN, 7
        BOTTOMMARGIN, 87
    END

    IDD_APPLIB_OPENCELL, DIALOG
    BEGIN
        LEFTMARGIN, 4
        RIGHTMARGIN, 215
        TOPMARGIN, 7
        BOTTOMMARGIN, 160
    END

    IDD_APPLIB_CREDENTIALS, DIALOG
    BEGIN
        LEFTMARGIN, 4
        RIGHTMARGIN, 193
        TOPMARGIN, 7
        BOTTOMMARGIN, 136
    END

    IDD_APPLIB_BADCREDS, DIALOG
    BEGIN
        LEFTMARGIN, 4
        RIGHTMARGIN, 331
        TOPMARGIN, 7
        BOTTOMMARGIN, 92
    END

    IDD_APPLIB_COVER, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 182
        TOPMARGIN, 7
        BOTTOMMARGIN, 128
    END

    IDD_APPLIB_BROWSE_FILESET, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 274
        TOPMARGIN, 7
        BOTTOMMARGIN, 143
        HORZGUIDE, 23
        HORZGUIDE, 107
    END

    IDD_APPLIB_BROWSE, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 274
        TOPMARGIN, 7
        BOTTOMMARGIN, 143
        HORZGUIDE, 23
        HORZGUIDE, 107
    END
END
#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Bitmap
//

IDB_LOCAL_USER          BITMAP  DISCARDABLE     "resource\\local_user.bmp"
IDB_LOCAL_GROUP         BITMAP  DISCARDABLE     "resource\\local_group.bmp"
IDB_FOREIGN_USER        BITMAP  DISCARDABLE     "resource\\foreign_user.bmp"
IDB_FOREIGN_GROUP       BITMAP  DISCARDABLE     "resource\\foreign_group.bmp"
IDB_ANY_OTHER           BITMAP  DISCARDABLE     "resource\\any_other.bmp"
IDB_FOREIGN_OTHER       BITMAP  DISCARDABLE     "resource\\foreign_other.bmp"
IDB_LOCAL_OTHER         BITMAP  DISCARDABLE     "resource\\local_other.bmp"
IDB_MASK_OBJ            BITMAP  DISCARDABLE     "resource\\mask_obj.bmp"
IDB_UNAUTHENTICATED     BITMAP  DISCARDABLE     "resource\\unauthenticated.bmp"

/////////////////////////////////////////////////////////////////////////////
//
// String Table
//

STRINGTABLE DISCARDABLE 
BEGIN
    IDS_BADCREDS_DESC_GENERAL 
                            "You do not have AFS tokens as an administrator within the %1 cell.\n\n%2\n\n%3"
    IDS_BADCREDS_DESC_EXPIRED 
                            "Your tokens within the %1 cell have expired.\n\n%2\n\n%3"
    IDS_BADCREDS_DESC_BADCHOICE 
                            "You have chosen not to log in as an administrator within the %1 cell.\n\n%2\n\n%3"
    IDS_BADCREDS_DESC2      "Unless you are logged in as an administrator, you may have insufficient privileges to work with the %1 cell."
    IDS_BADCREDS_DESC3      "Would you like to obtain new AFS tokens now?"
    IDS_BADCREDS_DESC3_ADM  "Would you like to log in as an administrator instead?"
    IDS_BADCREDS_DESC_DESTROYED 
                            "Your tokens within the %1 cell are no longer valid.\n\n%2\n\n%3"
END

STRINGTABLE DISCARDABLE 
BEGIN
    IDS_ERROR_BAD_CREDENTIALS 
                            "The AFS identity you have entered is unrecognized, or the password you have specified is incorrect."
    IDS_BROWSE_WAITING      "querying server--please wait..."
    IDS_BROWSE_BADCELL_TITLE "Unrecognized Cell"
    IDS_BROWSE_BADCELL_DESC "The cell name you have entered could not be found on the network.\n\nPlease check the cell name to ensure you have spelled it correctly."
    IDS_CREDS_NONE          "(no tokens)"
    IDS_CREDS_VALID         "%2, expires %3"
    IDS_CREDS_EXPIRED       "(tokens expired %3)"
END

#endif    // English (U.S.) resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

